//
// hello.HC-SR501.c
//
// HC-SR501 motion detector hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld 11/16/15
// (c) Massachusetts Institute of Technology 2015
//
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all
// liability.
//

#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#define serial_port PORTA
#define serial_direction DDRA
#define serial_pins PINA
#define serial_pin_in (1 << PA0)
#define serial_pin_out (1 << PA1)
#define serial_interrupt (1 << PCIE0)
#define serial_interrupt_pin (1 << PCINT0)

#define MISO PA5
#define MOSI PA6
#define miso_pin (1 << MISO)
#define mosi_pin (1 << MOSI)

#define extract_bit(c, i) ((c & (1 << i))) >> (i - 3)

#define node_id '4'

#include "core.h"

//ISR(PCINT0_vect) {
//  // pin change interrupt handler
//  static char chr;
//
//  get_char(&serial_pins, serial_pin_in, &chr);
//  if (chr == node_id) {
//    output(serial_direction, serial_pin_out);
//    put_char(&serial_port, serial_pin_out, 5);
//    put_char(&serial_port, serial_pin_out, p1);
//    input(serial_direction, serial_pin_out);
//    _delay_ms(10);
//  }
//}

int main(void) {
  // main
  // set clock divider to /1
  CLKPR = (1 << CLKPCE);
  CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

  // initialize serial pin
  set(serial_port, serial_pin_out);
  input(serial_direction, serial_pin_out);
//  output(serial_direction, serial_pin_out);

  set(serial_port, mosi_pin);
  output(serial_direction, mosi_pin);
  input(serial_direction, miso_pin);

  // set up pin change interrupt on input pin
//  set(GIMSK, serial_interrupt);
//  set (PCMSK0, serial_interrupt_pin);
//  sei();

  static char chr;
  static char slave_ids[3] = {'a', 'b', 'c'};
//  static char slave_ids[1] = {'b'};

  // main loop
  while (1) {
    static char chr;
    static unsigned char c1, c2, c3, p1, p2, p3;

    get_char(&serial_pins, serial_pin_in, &chr);
    int i;
    if (chr == node_id) {
      output(serial_direction, serial_pin_out);
      put_char(&serial_port, serial_pin_out, node_id);

      // query boards
      put_char(&serial_port, mosi_pin, 'a');
//      get_char(&serial_pins, miso_pin, &c1);
      get_char(&serial_pins, miso_pin, &p1);

      put_char(&serial_port, mosi_pin, 'b');
//      get_char(&serial_pins, miso_pin, &c2);
      get_char(&serial_pins, miso_pin, &p2);

      put_char(&serial_port, mosi_pin, 'c');
//      get_char(&serial_pins, miso_pin, &c3);
      get_char(&serial_pins, miso_pin, &p3);

      // calculate strings
      // string0
      if (extract_bit(p3, 3)) put_char(&serial_port, serial_pin_out, 6);
      else if (extract_bit(p3, 7)) put_char(&serial_port, serial_pin_out, 5);
      else if (extract_bit(p2, 3)) put_char(&serial_port, serial_pin_out, 4);
      else if (extract_bit(p2, 7)) put_char(&serial_port, serial_pin_out, 3);
      else if (extract_bit(p1, 3)) put_char(&serial_port, serial_pin_out, 2);
      else if (extract_bit(p1, 7)) put_char(&serial_port, serial_pin_out, 1);
      else put_char(&serial_port, serial_pin_out, 0);

      // string1
      if (extract_bit(p3, 2)) put_char(&serial_port, serial_pin_out, 6);
      else if (extract_bit(p3, 6)) put_char(&serial_port, serial_pin_out, 5);
      else if (extract_bit(p2, 2)) put_char(&serial_port, serial_pin_out, 4);
      else if (extract_bit(p2, 6)) put_char(&serial_port, serial_pin_out, 3);
      else if (extract_bit(p1, 2)) put_char(&serial_port, serial_pin_out, 2);
      else if (extract_bit(p1, 6)) put_char(&serial_port, serial_pin_out, 1);
      else put_char(&serial_port, serial_pin_out, 0);

      // string2
      if (extract_bit(p3, 1)) put_char(&serial_port, serial_pin_out, 6);
      else if (extract_bit(p3, 5)) put_char(&serial_port, serial_pin_out, 5);
      else if (extract_bit(p2, 1)) put_char(&serial_port, serial_pin_out, 4);
      else if (extract_bit(p2, 5)) put_char(&serial_port, serial_pin_out, 3);
      else if (extract_bit(p1, 1)) put_char(&serial_port, serial_pin_out, 2);
      else if (extract_bit(p1, 5)) put_char(&serial_port, serial_pin_out, 1);
      else put_char(&serial_port, serial_pin_out, 0);

      // string3
      if (extract_bit(p3, 0)) put_char(&serial_port, serial_pin_out, 6);
      else if (extract_bit(p3, 4)) put_char(&serial_port, serial_pin_out, 5);
      else if (extract_bit(p2, 0)) put_char(&serial_port, serial_pin_out, 4);
      else if (extract_bit(p2, 4)) put_char(&serial_port, serial_pin_out, 3);
      else if (extract_bit(p1, 0)) put_char(&serial_port, serial_pin_out, 2);
      else if (extract_bit(p1, 4)) put_char(&serial_port, serial_pin_out, 1);
      else put_char(&serial_port, serial_pin_out, 0);

      input(serial_direction, serial_pin_out);
    }


  }

//      get_char(&serial_pins, miso_pin, &p2);
//    get_char(&serial_pins, miso_pin, &p3);
//    get_char(&serial_pins, miso_pin, &p4);
//    get_char(&serial_pins, miso_pin, &p5);
//    get_char(&serial_pins, miso_pin, &p6);
//    get_char(&serial_pins, miso_pin, &p7);
//    get_char(&serial_pins, miso_pin, &p8);

}
