//
// hello.HC-SR501.c
//
// HC-SR501 motion detector hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld 11/16/15
// (c) Massachusetts Institute of Technology 2015
//
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all
// liability.
//

#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#define serial_port PORTA
#define serial_direction DDRA
#define serial_pins PINA
//#define serial_pin_in (1 << PA0)
//#define serial_pin_out (1 << PA1)
#define serial_interrupt (1 << PCIE0)
#define serial_interrupt_pin (1 << PCINT0)

#define MISO PA5
#define MOSI PA6
#define miso_pin (1 << MISO)
#define mosi_pin (1 << MOSI)

#define led_pin (1 << PA4)

#include "core.h"

static unsigned char pressed1;
static unsigned char pressed2;
static unsigned char pressed3;
static unsigned char pressed4;

static const unsigned char receive_pins[4] = {1 << PA0, 1 << PA1, 1 << PA2, 1 << PA3};

unsigned char scan_pad(volatile unsigned char *sendport, volatile unsigned char *sendddr, unsigned char sendpin,
                       unsigned char muxval) {

  static char result;
  // Wait for the receive pin to settle
  ADMUX = muxval;

  int temp_value = 100;
  while (temp_value > 10) {
    clear(PRR, 1 << PRADC);
    set(ADCSRA, 1 << ADSC);
    while (ADCSRA & (1 << ADSC));
    temp_value = ADCL;
    temp_value += ADCH << 8;
  }

  // Make receive pins inputs
  int i;
  for (i = 0; i < 4; i++) {
    clear(DDRA, receive_pins[i]);
  }

  // Activate the first column
  set(*sendddr, sendpin);
  set(*sendport, sendpin);

  // Get an analog read
  _delay_us(1);
  clear(PRR, 1 << PRADC);
  set(ADCSRA, 1 << ADSC);
  while (ADCSRA & (1 << ADSC));
  result = ADCH;
//  put_char(&serial_port, miso_pin, result);

  clear(*sendport, sendpin);
  clear(*sendddr, sendpin);

  // Make receive pins outputs to quickly bleed off charge.
  for (i = 0; i < 4; i++) {
    set(DDRA, receive_pins[i]);
  }

//  _delay_ms(1);
  return result;
}

void read_touchpads(void) {
  // Had to move constant definitions here because compiler was complaining
  static const unsigned char send_pins[2] = {1 << PA7, 1 << PB2};
  static volatile unsigned char* send_ports[2] = {&PORTA, &PORTB};
  static volatile unsigned char* send_ddrs[2] = {&DDRA, &DDRB};
  static const unsigned char receive_muxes[4] = {0b000000, 0b000001, 0b000010, 0b000011};

  // connect to MISO
//  set(serial_port, MISO);
//  set(serial_direction, MISO);
  output(serial_direction, miso_pin);

  // read from the touchpads
  // Sometimes, the h goes back too fast, so delay for a little bit
//  _delay_ms(10);

  // send id
//  put_char(&serial_port, miso_pin, node_id);

  unsigned char result = 0b0;

  // read and send output
  result |= (scan_pad(send_ports[0], send_ddrs[0], send_pins[0], receive_muxes[0]) < THRESHOLD_0) << 7;
  result |= (scan_pad(send_ports[0], send_ddrs[0], send_pins[0], receive_muxes[1]) < THRESHOLD_1) << 6;
  result |= (scan_pad(send_ports[0], send_ddrs[0], send_pins[0], receive_muxes[2]) < THRESHOLD_2) << 5;
  result |= (scan_pad(send_ports[0], send_ddrs[0], send_pins[0], receive_muxes[3]) < THRESHOLD_3) << 4;
  result |= (scan_pad(send_ports[1], send_ddrs[1], send_pins[1], receive_muxes[0]) < THRESHOLD_4) << 3;
  result |= (scan_pad(send_ports[1], send_ddrs[1], send_pins[1], receive_muxes[1]) < THRESHOLD_5) << 2;
  result |= (scan_pad(send_ports[1], send_ddrs[1], send_pins[1], receive_muxes[2]) < THRESHOLD_6) << 1;
  result |= (scan_pad(send_ports[1], send_ddrs[1], send_pins[1], receive_muxes[3]) < THRESHOLD_7) << 0;

  put_char(&serial_port, miso_pin, result);

  // disconnect from MISO
  input(serial_direction, miso_pin);
}

//void flash() {
//  // LED flash delay
//  clear(serial_port, led_pin);
//  _delay_ms(100);
//  set(serial_port, led_pin);
//}

int run(void) {
  // main
  // set clock divider to /1
  CLKPR = (1 << CLKPCE);
  CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

  // initialize serial pin
//  set(serial_port, serial_pin_out);
//  output(serial_direction, serial_pin_out);
  set(serial_port, miso_pin);
  input(serial_port, miso_pin);
  input(serial_port, mosi_pin);

  set(serial_port, led_pin);
  output(serial_direction, led_pin);


  // init A/D
//  ADCSRA = (1 << ADEN) // enable
//           | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128

  // ADC enabled, flag cleared, prescalar set to 128 (125 kHz)
  ADCSRA = (1 << ADEN) | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0);
  ADCSRB |= (1 << ADLAR);

  // set up pin change interrupt on input pin
//  set(GIMSK, serial_interrupt);
//  set (PCMSK0, serial_interrupt_pin);
//  sei();

  static char chr;

  // main loop
  while (1) {
    get_char(&serial_pins, mosi_pin, &chr);
    if (chr == node_id) {
      // If we hear our name, it's our turn to talk back to the master.
      read_touchpads();
    }
  }
}
